using System;
using System.Collections;

namespace CustomTextParser
{
	/// <summary>
	/// A basic text parser, looks for some common names and suggests
	/// a shorter version.
	/// </summary>
	public class CustomParser : Keyoti.RapidSpell.CheckerEngineAdapter
	{
		Hashtable dictionary = new Hashtable();
		String theText, suggestion = null;
		
		Keyoti.RapidSpell.FrameWorkSpecific.AdvancedTextBoundary wordIterator = new Keyoti.RapidSpell.FrameWorkSpecific.AdvancedTextBoundary();
		int wordStart, wordEnd;

		public CustomParser(){
			//load dictionary of long and short names
			dictionary.Add("Andrew", "Andy");
			dictionary.Add("Charles", "Charlie");
			dictionary.Add("David", "Dave");			
			dictionary.Add("Michael", "Mike");
			dictionary.Add("Rebecca", "Becky");
			dictionary.Add("Elizabeth", "Beth");
			dictionary.Add("Nicole", "Nick");
			dictionary.Add("James", "Jim");
		}

		public override void Check(String text){
			theText = text;
			wordIterator.SetText(text);
		}


		public override Keyoti.RapidSpell.BadWord NextBadWord()
		{
			string nextWord = GetNextWord();
			int currentBadWordStart = -1;
			int currentBadWordEnd = -1;
			if (nextWord==null) return null;
			else 
			{

				//see if next word is bad or if we are to ignore this word, if not scan for next bad word 
				while(!dictionary.Contains(nextWord))
				{
					nextWord = GetNextWord();
					
					//ran out of words
					if (nextWord==null) return null;


					
				}
				currentBadWordStart = wordStart;
				currentBadWordEnd = wordEnd;
				if (dictionary.ContainsKey(nextWord))
				{
					suggestion = (String) dictionary[nextWord];	
				}
			}
			return new Keyoti.RapidSpell.BadWord(nextWord,currentBadWordStart,currentBadWordEnd);
		}

		

		/// Returns the next word in the text.
		/// <returns>String the next word in the text</returns>
		internal String GetNextWord() 
		{

			wordStart = wordEnd;
			if(wordEnd < wordIterator.Last()) 
			{
				wordEnd = wordIterator.Following( wordEnd );

				if(wordEnd-wordStart <= 0 
					|| !Char.IsLetterOrDigit( theText[wordStart] ) 
					)
				{

					//ignore and move on to next word
					return GetNextWord();
				}

				//return as a word
				return theText.ToString().Substring(wordStart, wordEnd - wordStart);
			} 
			else 
			{
				return null;
			}
		}




		public override ArrayList FindSuggestions(){
			ArrayList s = new ArrayList();
			if (suggestion != null) s.Add(suggestion);
			return s;
		}
	}
}
